from rich.markdown import Markdown
from rich.console import Console
from rich.text import Text
from langchain_openai import ChatOpenAI
import re
import subprocess

from nucleus.logger import log

console = Console()

class MessagePrinter:
    def __init__(self):
        self.console = Console()
        self.style="#fafcfb"

    def system_message(self, message):
        """
        Prints a system message with specific styling.
        """
        system_text = Text(message, style="yellow")
        self.console.print(system_text)

    def user_message(self, message):
        """
        Prints a user message with Markdown rendering.
        """
        user_text = Text(f"{message}")
        self.console.print(user_text, style=self.style)

    def assistant_message(self, message, type=None):
        """
        Prints an assistant message with optional Markdown rendering.
        """
        if type=='command':
            print("\n"+ message)
        else:
            # pprint(message)
            assistant_text = Markdown(f"<br>{message}")
            self.console.print(assistant_text)

def command_provider(message , model_name):
    """
    """

    llm = None

    if model_name == 'openai':
        llm = ChatOpenAI(
                model="gpt-4o",
                temperature=0,
                max_tokens=None,
                timeout=None,
                max_retries=2
            )
        
    if llm is not None:
        message = f"Be precise in your response. Here is the question: {message}"
        response = llm.invoke(message)
    else:
        return "No LLM chosen"
    
    return response.content

def format_message(message, role):
    """
    """
    if role=='user':
        return (
            'human', message
        )
    else:
        return (
            'assistant', message
        )

def get_response(response, message_printer):

    try:
        # print_response(response)

        message_printer.assistant_message(response)
   
        command = extract_command(response)
        if command:
            # ask
            if confirm_ask():
                # print_response(command)
                result = subprocess.run(command, shell=True, capture_output=True, text=True)

                if result.returncode==0:
                    
                    # print_response(f"command executed \n {result.stdout}", type='command')
                    message_printer.assistant_message(f"command executed \n {result.stdout}", type='command')
                else:
                    message_printer.assistant_message(result.stderr, type='command')
                    # print_response(result.stderr, type='command')
    except Exception as e:
        log.error("Error in get_response", e)

def confirm_ask():
    """
    Prompt the user to decide whether to run or not.
    """
    ask_text = "\n[bold cyan]Do you want to run?[/bold cyan] [green](Yes or No):[/green] "
    # console.print(ask_text, end=" ")
    response = console.input(ask_text).strip().lower()
    if response in ['yes', 'y']:
        return True
    elif response in ['no', 'n']:
        return False
    else:
        print("Invalid input. Please respond with 'Yes' or 'No'.")
        confirm_ask()  # Re-prompt the user

def extract_command(text):
    """
    Extracts a command from a text block surrounded by triple backticks and in bash syntax.
    
    Args:
        text (str): The input text containing the command.
        
    Returns:
        str: The extracted command, or None if no command is found.
    """
    match = re.search(r"```bash\n(.+?)```", text, re.DOTALL)
    if match:
        return match.group(1).strip()
    return None

# Example usage
if __name__ == "__main__":
    printer = MessagePrinter()

    # Print system messages
    printer.print_system_message("Type commands as usual. Ask anything you want.")
    printer.print_system_message("Type 'exit' or 'quit' to stop the program.\n")

    # Print user message
    printer.print_user_message("Hello, how do I use this program?")

    # Print assistant message
    printer.print_assistant_message("Sure! Let me explain how it works.")

    # Print messages with Markdown rendering
    printer.print_user_message("### This is a Markdown heading")
    printer.print_assistant_message("**This text is bold in Markdown**")





import os
from flask import Flask, cli, send_from_directory
from flask_cors import CORS
from flask_socketio import SocketIO
import logging
import time
from threading import Thread
from pkg_resources import resource_filename
from multiprocessing import Queue

# Disable flask logging
log = logging.getLogger('werkzeug')

log.setLevel(logging.ERROR)
log.disabled = True

cli.show_server_banner = lambda *_: None

frontend_path = resource_filename(__name__, 'visualization/dist')

app = Flask(__name__, static_folder='visualization/dist')


UPLOAD_FOLDER = os.path.abspath(os.path.dirname(__file__))

app.config["UPLOAD_FOLDER"] = UPLOAD_FOLDER
CORS(app)  # Allow CORS for all origins

socketio = SocketIO(app, cors_allowed_origins="*",  async_mode="threading")

@app.route('/uploads/<path:name>')
def download_file(name):
    """Endpoint to download files."""
    return send_from_directory(app.config["UPLOAD_FOLDER"], name)

# Serve static files (e.g., JS, CSS)
@app.route('/assets/<path:filename>')
def static_files(filename):
    return send_from_directory(os.path.join(app.static_folder, 'assets'), filename)

# Serve the React app
@app.route('/', defaults={'path': ''})
@app.route('/<path:path>')
def serve(path):
    if path != "" and os.path.exists(os.path.join(app.static_folder, path)):
        return send_from_directory(app.static_folder, path)
    return send_from_directory(app.static_folder, 'index.html')


@socketio.on("data")
def send_data():
    """
    """
    while True:
        time.sleep(5)  # Wait for 5 seconds before sending an update
        if not data_queue.empty():
            data = data_queue.get()
            socketio.emit('data', {'config': data})

def run_flask_app(queue):
    """
    """
    global data_queue
    data_queue = queue

    Thread(target=send_data, daemon=True).start()
      
    socketio.run(app, port=5000, debug=False, allow_unsafe_werkzeug=True, log_output=False)

if __name__ == '__main__':
    # global data_queue
    data_queue = Queue()
    data_queue.put({"message": "hello"})
    
    Thread(target=send_data, daemon=True).start()
    socketio.run(app, port=5000, debug=True)




from multiprocessing import Process, Event, Queue
import os
import subprocess
import shlex
import argparse
import re
import random
import subprocess

from nucleus.data_viz.planner import plan
from nucleus.terminal.suggestion import session, PLACEHOLDER
from nucleus.gui import run_flask_app
from nucleus.terminal.planner import QueryManager
from nucleus.logger import log
from nucleus.tools import MessagePrinter

def extract_command(text):
    """
    Extracts a command from a text block surrounded by triple backticks and in bash syntax.
    
    Args:
        text (str): The input text containing the command.
        
    Returns:
        str: The extracted command, or None if no command is found.
    """
    match = re.search(r"```bash\n(.+?)```", text, re.DOTALL)
    if match:
        return match.group(1).strip()
    return None


def confirm_ask():
    """
    Prompt the user to decide whether to run or not.
    """
    ask_text = "\n[bold cyan]Do you want to run?[/bold cyan] [green](Yes or No):[/green] "
    # console.print(ask_text, end=" ")
    response = console.input(ask_text).strip().lower()
    if response in ['yes', 'y']:
        return True
    elif response in ['no', 'n']:
        return False
    else:
        print("Invalid input. Please respond with 'Yes' or 'No'.")
        confirm_ask()  # Re-prompt the user


def args_parser():
    parser = argparse.ArgumentParser()
    parser.add_argument("--openai-api-key",  help="Specify the OpenAI API key")
    parser.add_argument("--anthropic-api-key", help="Specify the Anthropic API key")
    args = parser.parse_args()

    model, api_key = None, None

    input_vals = {
        'LLM': []
    }

    if args.openai_api_key:
        model = 'openai'
        api_key = args.openai_api_key

        input_vals['LLM'] = [{
            'model': model, 
            "api_key": api_key
            }]
        
        return input_vals
    
    if args.anthropic_api_key:
        model = "anthropic"
        api_key = args.anthropic_api_key

        input_vals['LLM'] = [{
            'model': model, 
            "api_key": api_key
            }]
        
        return input_vals

    openai_api_key = os.getenv("OPENAI_API_KEY")
    anthropic_api_key = os.getenv("ANTHROPIC_API_KEY")
    gemini_api_key = os.getenv("GEMINI_API_KEY")

    if openai_api_key:
        model='openai'
        api_key = openai_api_key
        
        input_vals['LLM'].append({
            "model":model,
            "api_key": api_key
        })
    
    if anthropic_api_key:
        model='anthropic'
        api_key = anthropic_api_key

        input_vals['LLM'].append({
            "model":model,
            "api_key": api_key
        })
    
    if gemini_api_key:
        model='gemini'
        api_key = gemini_api_key

        input_vals['LLM'].append({
            "model":model,
            "api_key": api_key
        })
    
    return input_vals
    
def main():

    input_vals = {}

    input_args = args_parser()
    if not len(input_args['LLM']):
        log.error("Error: No API key provided. Please provide an API key to proceed.")
        return

    if len(input_args['LLM'])==1:
        input_vals['LLM'] = input_args['LLM'][0]

    if len(input_args["LLM"])>1:
        log.warning("Multiple API keys found in your environment variables, but none explicitly provided.")
        llm = random.choice(input_args["LLM"])
        log.info(f"Randomly selected API key for the model '{llm['model']}'.")
        input_vals['LLM'] = llm


    message_printer = MessagePrinter()

    message_printer.system_message("Type commands as usual. ask anything u want")
    message_printer.system_message("Type 'exit' or 'quit' to stop the program.\n")

    ## run flask local server for visualization
    data_queue = Queue()
    flask_process = Process(target=run_flask_app, args=(data_queue,))
    flask_process.start()

    query_responder = QueryManager(input_vals, message_printer)
    while True:
        try:
            # Prompt for user input

            user_input = session.prompt("\n> ", placeholder=PLACEHOLDER)

            # Check if the user wants to exit
            if user_input.lower() == "":
                continue

            if user_input.lower() in ["exit", "quit"]:
                message_printer.system_message("Exiting shell. Goodbye!")
                                
                flask_process.terminate()
                flask_process.join()

                break

            if user_input.lower()[0] in "!":
                # handle 'cd' command to change directory
                user_input = user_input[1:]
                if user_input.startswith("cd"):
                    parts = shlex.split(user_input)
                    if len(parts) > 1:
                        new_dir = parts[1]
                    else:
                        new_dir = os.path.expanduser("~")
                    try:
                        os.chdir(new_dir)
                    except FileNotFoundError:
                        print(f"cd: {new_dir}: No such file or directory")
                    except Exception as e:
                        print(f"cd: {e}")
                else:
                    subprocess.run(shlex.split(user_input), check=False)

            elif user_input.startswith("/show"):
                file_input = user_input.split("/show", 1)[-1].strip()
                config = plan(file_input, session)
                data_queue.put(config)

                message_printer.system_message("You can view the file at http://127.0.0.0:5000")

            elif user_input == "/close":
                if flask_process and flask_process.is_alive():
                    message_printer.system_message("Stopping visualization process...")
                    flask_process.terminate()
                    flask_process.join()
                # Run other normal terminal commands
                else:
                    subprocess.run(shlex.split(user_input), check=False)
            else:
                query_responder.execute_query(user_input)

        except KeyboardInterrupt:
            message_printer.system_message("\n Exiting shell. Goodbye!")
            break
        except Exception as e:
            log.error("Error in main : {e}")
    
    flask_process.terminate()
    flask_process.join()



if __name__ == "__main__":
    main()



import sys

def model_prompt():
    return (
        "system",
        """You are a helpful assistance. Help user to write biotools commands based the query.
            if the user types a terminal command then respond with the same command bash markdown block.
            Here's an example text.
                1. USER convert haplotype.sam file to haplotype.bam 
                    ASSISTANT  ```bash samtools view -S -b haplotype.sam > haplotype.bam```
                2 USER ls
                    ASSISTANT ```bash ls ```
            if you do not understand the message or the query, ask the user!
        """
        )

def planner_prompt():
    """
    """
    return [(
                "system",
                """
                You are an export query planner capable of breaking apart questions into its depenencies queries such that the answers can be used to respond to the parent question. \
                    The queries can be two kinds:
                        1. It can be related to terminal commands to perform a task using bioinformatic tools. \
                        2. general question related to bioinformatics or biology. 

                        If the query is related to commands, your goal is to provide the command so that user can further execute that command in the terminal.
                    - Do not answer the questions, simply provide correct plan with subquestion if necessary to in order to respond to the main question. \
                    - Before you call the function, think step by step to get a better understanding of the problem. \
                    - List the subqueries from lowest dependecy to highest dependency. 
                
                If you do not understand the query ask the user back. Always design small plans. 
                """,
            ),
            (
                "user",
                "Consider: {question}\nGenerate the correct query plan. \
                    If the query has NO dependency of other subqueries, then it is SINGLE_QUESTION query_type, else it is MULTI_DEPENDENCY.",
            )
        ]
